#ifndef _FileInfoDlg_CPP
#define _FileInfoDlg_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "../Resources/Resource.H"

#include "../Source/NSWFL.H"
#include "../Source/Package.H"
#include "../Source/Entry.H"
#include "../Source/Common.H"

#include "../Dialogs/MainDlg.H"
#include "../Dialogs/FileInfoDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//---------------------(Variable Declarations)
HWND FileInfoDialog_hWnd = NULL;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

BOOL CALLBACK FileInfoDialog(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    //--------------------------------------------------------------------------

    static HMENU SystemMenu = NULL;

	static HWND hName = NULL;
	static HWND hType = NULL;
	static HWND hPath = NULL;
	static HWND hFSize = NULL;
	static HWND hPSize = NULL;
	static HWND hRatio = NULL;
	static HWND hLastMod = NULL;
	static HWND hCRC32 = NULL;
	static HWND hAttrib = NULL;

    //--------------------------------------------------------------------------

    if(uMsg == WM_INITDIALOG) // Received A Initilize Dialog Message
    {
        FileInfoDialog_hWnd = hWnd;

        SystemMenu = GetSystemMenu(hWnd, FALSE);
        AppendMenu(SystemMenu, MF_SEPARATOR, 0, 0);
        AppendMenu(SystemMenu, MF_STRING, 1000, "About");

        SendMessage(hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)DISPLAYACRONYM"Information");
        SendMessage(hWnd, WM_SETICON, TRUE, (LPARAM) LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN)) );

		hName		= GetDlgItem(hWnd, IDC_NAME);
		hType		= GetDlgItem(hWnd, IDC_TYPE);
		hPath		= GetDlgItem(hWnd, IDC_PATH);
		hFSize		= GetDlgItem(hWnd, IDC_FILESIZE);
		hPSize		= GetDlgItem(hWnd, IDC_PACKSIZE);
		hRatio		= GetDlgItem(hWnd, IDC_RATIO);
		hLastMod	= GetDlgItem(hWnd, IDC_LASTMOD);
		hCRC32		= GetDlgItem(hWnd, IDC_CRC32);
		hAttrib		= GetDlgItem(hWnd, IDC_ATTRIB);

		SendMessage(hName, EM_SETREADONLY, 1, 0);
		SendMessage(hType, EM_SETREADONLY, 1, 0);
		SendMessage(hPath, EM_SETREADONLY, 1, 0);
		SendMessage(hFSize, EM_SETREADONLY, 1, 0);
		SendMessage(hPSize, EM_SETREADONLY, 1, 0);
		SendMessage(hRatio, EM_SETREADONLY, 1, 0);
		SendMessage(hLastMod, EM_SETREADONLY, 1, 0);
		SendMessage(hCRC32, EM_SETREADONLY, 1, 0);
		SendMessage(hAttrib, EM_SETREADONLY, 1, 0);

		char sBuf[2048];

		int iItem = ListView_GetNextItem(hFileList, -1, LVNI_SELECTED);	

		if(iItem >= 0)
		{
			ListView_GetItemText(hFileList, iItem, 0, sBuf, sizeof(sBuf));
			Set_Text(hName, sBuf);

			ListView_GetItemText(hFileList, iItem, 1, sBuf, sizeof(sBuf));
			Set_Text(hType, sBuf);

			ListView_GetItemText(hFileList, iItem, 2, sBuf, sizeof(sBuf));
			Set_Text(hLastMod, sBuf);

			ListView_GetItemText(hFileList, iItem, 3, sBuf, sizeof(sBuf));
			Set_Text(hFSize, sBuf);

			ListView_GetItemText(hFileList, iItem, 4, sBuf, sizeof(sBuf));
			Set_Text(hRatio, sBuf);

			ListView_GetItemText(hFileList, iItem, 5, sBuf, sizeof(sBuf));
			Set_Text(hPSize, sBuf);

			ListView_GetItemText(hFileList, iItem, 6, sBuf, sizeof(sBuf));
			Set_Text(hCRC32, sBuf);

			ListView_GetItemText(hFileList, iItem, 7, sBuf, sizeof(sBuf));
			Set_Text(hAttrib, sBuf);

			ListView_GetItemText(hFileList, iItem, 8, sBuf, sizeof(sBuf));
			Set_Text(hPath, sBuf);
		}

        CenterWindow(hWnd);

		return TRUE; // Return TRUE to set the keyboard focus, Otherwise return FALSE
    }

    //--------------------------------------------------------------------------

    if(uMsg == WM_COMMAND)
    {
        if(wParam == IDC_CMDOK)
        {
	        EndDialog(hWnd, 0);
			DestroyWindow(hWnd);
            return TRUE;
        }

		return FALSE;
    }

    //--------------------------------------------------------------------------

	if(uMsg == WM_PAINT)
    {
        HDC ThisHDC;
        PAINTSTRUCT ThisPS;

        ThisHDC = BeginPaint(hWnd, &ThisPS);

        // Any painting should be done here

        EndPaint(hWnd, &ThisPS);
        return TRUE;
    }

    //--------------------------------------------------------------------------

    if(uMsg == WM_SYSCOMMAND) // Received A System Menu Message
    {
        if(LOWORD(wParam) == 1000) //- About (SysMenu Command)
        {
            _AboutDialogInfo ADI;
    
            ADI.DisplayIcon  = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_MAIN));
            ADI.TitleCaption = gsTitleCaption;
            ADI.FileVersion  = gsFileVersion;
            ADI.BuildDate    = __DATE__;
            ADI.BuildTime    = __TIME__;
            ADI.CopyRight    = gsAppCopyRight;
            ADI.OwnerHandle  = hWnd;

            NetLogo(&ADI);
            return TRUE;
        }

        return FALSE;
    }

    //--------------------------------------------------------------------------

    if(uMsg == WM_CLOSE)  // Received Close Message
    {
        EndDialog(hWnd, 0);
        DestroyWindow(hWnd);
        return TRUE;
    }

    //--------------------------------------------------------------------------

    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif

