#ifndef _CREGCODE_CPP_
#define _CREGCODE_CPP_
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CRegCode.H"

#include "../../Dialogs/RegistrationDlg.H"


#include "../CStatusDlg/CStatusDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CRegCode::MakeHex(const unsigned char *sBuf, const int iBufSz, unsigned char *sOutCode, int iMaxSz)
{
	char sHex[3];
	int iRPos = 0;
	int iLen = 0;

	strcpy_s((char *)sOutCode, iMaxSz, "");

	while(iRPos < iBufSz)
	{
		sprintf_s((char *)sHex, sizeof(sHex), "%X", sBuf[iRPos]);

		if(sBuf[iRPos] < 16)
		{
			strcat_s((char *)sOutCode, iMaxSz, "0");
		}

		strcat_s((char *)sOutCode, iMaxSz, sHex);

		iLen = (iLen + 2);

		iRPos++;
	}

	 return iLen;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CRegCode::MakeRegHeyHash(const unsigned char *sRawHey, int iRawKeySz, unsigned char *sOutHash)
{
	int iRPos = 0;
	int iWPos = 0;
	int iKeySum = 0;

	while(iRPos < iRawKeySz)
	{
		sOutHash[iWPos++] = sRawHey[iRPos++];
	}

	iRPos = 0;
	iWPos = 0;

	while(iRPos < iRawKeySz)
	{
		iKeySum = (iKeySum + sOutHash[iRPos++]);
	}

	iWPos = 0;
	while(iWPos < iRawKeySz)
	{
		sOutHash[iWPos] = (sOutHash[iWPos] + sRawHey[(iRawKeySz - iWPos) - 1]) + iKeySum;
		iWPos++;
	}

	return iRawKeySz;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CRegCode::GenCode(const unsigned char *sKey, const int iKeyLen, int *iKeySeed, const int iGenLen, unsigned char *sOutCode)
{
	int iKeyPos = 0;
	int iWPos = 0;
	int iSeed = *iKeySeed;
	int iLastVal = sKey[0];

	while(iWPos < iGenLen)
	{
		sOutCode[iWPos] = (sKey[iKeyPos] + ((iLastVal ^ sKey[iKeyPos]) * sKey[iKeyPos])) ^ iSeed;

		if(sOutCode[iWPos] < 0)
		{
			sOutCode[iWPos] = (0 - sOutCode[iWPos]);
		}

		iLastVal = sOutCode[iWPos];

		if(iKeyPos == iKeyLen)
		{
			iKeyPos = -1;
		}

		iKeyPos++;
		iWPos++;
		iSeed++;
	}

	*iKeySeed = iSeed;

	sOutCode[iWPos] = '\0';

	return iWPos;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int CRegCode::FormatRegCode(const unsigned char *sRegCode, int iRegCodeLen, const int iDashSep, unsigned char *sOutRegCode)
{
	int iRPos = 0;
	int iWPos = 0;
	int iDashPos = 0;

	while(iRPos < iRegCodeLen)
	{
		if(iDashPos == iDashSep)
		{
			sOutRegCode[iWPos++] = '-';
			iDashPos = 0;
		}

		sOutRegCode[iWPos++] = sRegCode[iRPos++];
		
		iDashPos++;
	}

	sOutRegCode[iWPos] = '\0';

	return iWPos;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CRegCode::GenRegCodes(const unsigned char *snKeySeed, int *iKeySeed, int iRegCodes, 
						  int iRegCodeLen, int iDashSpacing, const char *sOutputFileName)
{
	unsigned char sRawCode[1024];
	unsigned char sHexCode[1024];
	unsigned char sKeyHash[1024];
	unsigned char sFormatRegCode[1024];

	int iCodeSz = 0;
	int iRedCode = 0;
	int iKeyLen = 0;
	int iHexLen = 0;

	FILE *hTarget = NULL;

	if(fopen_s(&hTarget, sOutputFileName, "wb") != 0)
	{
		return false;
	}

	//Generate a reg-code seed key.
	iKeyLen = MakeRegHeyHash(snKeySeed, strlen((char *)snKeySeed), sKeyHash);

	while(iRedCode < iRegCodes)
	{
		//Generate a binary reg-code.
		iCodeSz = GenCode(sKeyHash, iKeyLen, iKeySeed, iRegCodeLen, sRawCode);

		//Make a Hex Reg-code from the binary reg-code.
		iHexLen = MakeHex(sRawCode, iCodeSz, sHexCode, sizeof(sHexCode));

		//Generate a new reg-code seed key using the hex reg-code.
		iKeyLen = MakeRegHeyHash(sHexCode, iHexLen, sKeyHash);

		//Format the hex reg-code (add dashes). 
		iCodeSz = FormatRegCode(sHexCode, iHexLen, iDashSpacing, sFormatRegCode);

		strcat_s((char *)sFormatRegCode, sizeof(sFormatRegCode), "\r\n");
		fwrite(sFormatRegCode, sizeof(char), iCodeSz + 2, hTarget);

		iRedCode++;
	}

	fclose(hTarget);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CRegCode::CheckRegCode(const unsigned char *snKeySeed, int *iKeySeed, int iRegCodes, int iRegCodeLen, const unsigned char *sInCompareKey)
{
	unsigned char sRawCode[1024];
	unsigned char sHexCode[1024];
	unsigned char sKeyHash[1024];
	unsigned char sCompareKey[1024];

	bool bResult = false;
	int iCodeSz = 0;
	int iRedCode = 0;
	int iKeyLen = 0;
	int iHexLen = 0;
	int iRPos = 0;
	int iWPos = 0;

	CStatusDlg MyStatDlg;

	MyStatDlg.Load(RegistrationDialog_hWnd, "Checking Registration", 0, true);

	MyStatDlg.Show();

	MyStatDlg.SetProgressRange(0, iRegCodes);

	while(sInCompareKey[iRPos] != 0)
	{
		if(sInCompareKey[iRPos] == '-')
		{
			iRPos++;
		}
		else sCompareKey[iWPos++] = sInCompareKey[iRPos++];
	}

	sCompareKey[iWPos] = '\0';

	//Generate a reg-code seed key.
	iKeyLen = MakeRegHeyHash(snKeySeed, strlen((char *)snKeySeed), sKeyHash);

	while(iRedCode < iRegCodes)
	{
		//Generate a binary reg-code.
		iCodeSz = GenCode(sKeyHash, iKeyLen, iKeySeed, iRegCodeLen, sRawCode);

		//Make a Hex Reg-code from the binary reg-code.
		iHexLen = MakeHex(sRawCode, iCodeSz, sHexCode, sizeof(sHexCode));

		//Generate a new reg-code seed key using the hex reg-code.
		iKeyLen = MakeRegHeyHash(sHexCode, iHexLen, sKeyHash);

		if(_strcmpi((char *)sCompareKey, (char *)sHexCode) == 0)
		{
			bResult = true;
			break;
		}

		if(!MyStatDlg.IsDlg())
		{
			break;
		}
		
		MyStatDlg.SetProgressPos(iRedCode);
		iRedCode++;
	}

	if(bResult)
	{
		MyStatDlg.SetProgressPos(iRegCodes);
		MyStatDlg.SetText("Registration Code Authenticated. Thank You!");
		MyStatDlg.SetButtonText("Ok");
	}
	else{
		MyStatDlg.SetProgressPos(iRegCodes);
		MyStatDlg.SetText("The Registration Code is Invalid!");
		MyStatDlg.SetButtonText("Ok");
	}

	while(MyStatDlg.IsDlgVisible())
	{
		Sleep(1);
	}

	MyStatDlg.Unload();
	
	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
