#ifndef _ROUTINES_CPP
#define _ROUTINES_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "NSWFL.H"
#include "Debug.H"
#include "Init.H"
#include "Entry.H"
#include "Routines.H"
#include "CRC32.H"

#include "NASCCL/NASCCL.H"

#define APP_IDENTITY           "Project Epsilon Certificate"
#define COMP_VERSION           "Project Epsilon Certificate Compatibility Version 000100x"
#define THUMB_PRINT_SZ         128
#define RESERVE_HEADER_SZ      1024
#define CHECKSUMLEN            1024

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

long Reflect_Checksum(long lReflect, char cChar)
{
	long Value = 0;

	// Swap Bit 0 For Bit 7 Bit 1 For Bit 6, Etc....
	for(int Loop = 1; Loop < (cChar + 1); Loop++)
	{
		if(lReflect & 1)
        {
            Value |= 1 << (cChar - Loop);
        }

		lReflect >>= 1;
	}

	return Value;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

long MakeChecksum(const char *sRawKey)
{
    char sKeyHash[CHECKSUMLEN + 1];
    int iRPos = 0;
    long lChecksum = 0;

	NASCCL MyHash;

    MyHash.HashKey(sRawKey, sKeyHash, strlen(sRawKey), CHECKSUMLEN);

	while(iRPos < CHECKSUMLEN)
    {
        lChecksum = (lChecksum + sKeyHash[iRPos]) + Reflect_Checksum(lChecksum, sKeyHash[iRPos]);
        iRPos++;
    }

    return lChecksum;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD EncryptCertKey(DWORD dwINFinalCRC, HANDLE hfTarget, NASCCL *lpCode, int iCertKeySz)
{
	DWORD dwFinalCRC = dwINFinalCRC;

	//Generate the certificate key.
	char sCertKey[CERTKEYBUFSZ + 1];

	DWORD dwKeySzLeft = (DWORD)iCertKeySz;
	DWORD dwBytesWritten = 0;
	DWORD dwBufSz = CERTKEYBUFSZ;

	int iCryptTemp = 0;
	int iSeed = GetTickCount();

	//Encrypt and write the certificate key size.
	lpCode->CipherEx(&iCertKeySz, &iCryptTemp, sizeof(iCryptTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);

	while(dwKeySzLeft > 0)
	{
		if(dwKeySzLeft <= dwBufSz)
		{
			dwBufSz = dwKeySzLeft;
		}

		GenerateKey(dwBufSz, iSeed, GKNONREPETITION | GKBROADKEY, sCertKey);

		dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&sCertKey, dwBufSz);

		lpCode->CipherEx(sCertKey, sCertKey, dwBufSz);		

		WriteFile(hfTarget, sCertKey, dwBufSz, &dwBytesWritten, NULL);

		dwKeySzLeft -= dwBufSz;
		iSeed += dwKeySzLeft;
	}

	return dwFinalCRC;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool WriteCert(HWND hWnd, const char *sTarName, const char *sUserKey,
			   int iCertAttribs, int iCryptAlgo, int iCertKeySz,
			   const char *sFirstName, const char *sMiddleName, const char *sLastName,
			   const char *sAddress1, const char *sAddress2,
			   const char *sCity, const char *sState, const char *sZipCode, const char *sCountry,
			   const char *sHomePhone, const char *sWorkPhone, const char *sCellPhone, const char *sFaxNumber,
			   const char *sEmailAddress, const char *sComments)
{
	SYSTEMTIME stCreation;

	DWORD dwCreationDate = 0;
	DWORD dwCreationTime = 0;
	DWORD dwBytesWritten = 0;
	DWORD dwFinalCRC = 0xffffffff;
	long lChecksum = 0;

	int iTemp = 0;
	int iCryptTemp = 0;

	char sCryptTemp[1100];
	char sStatusText[1024];
	char sThumbPrint[THUMB_PRINT_SZ + 1];

	NASCCL DfltCode;

	memset(&DfltCode, 0, sizeof(DfltCode));

	HANDLE hfTarget = CreateFile(
		sTarName,
		GENERIC_WRITE,
		0,
		NULL,
		CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL,
		NULL
	);

	if(hfTarget == INVALID_HANDLE_VALUE)
    {
        sprintf_s(sStatusText, sizeof(sStatusText),
			"Failed to open write file:\r\n'%s'",
			sTarName);

		MessageBox(hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);

		CloseHandle(hfTarget);

        return false;
    }

	//------------------------------------------------------------------

	DfltCode.InitializeCryptography(sUserKey);

	GetSystemTime(&stCreation);

	dwCreationDate = DateToLong(stCreation.wYear, stCreation.wMonth, stCreation.wDay);
	dwCreationTime = TimeToLong(stCreation.wHour, stCreation.wMinute, stCreation.wSecond);

	GenerateKey(THUMB_PRINT_SZ, dwCreationDate + dwCreationTime, GKNONREPETITION | GKBROADKEY, sThumbPrint);

	//------------------------------------------------------------------

	//Write the Application Checksum.
	lChecksum = MakeChecksum(APP_IDENTITY);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&lChecksum, sizeof(lChecksum));
	WriteFile(hfTarget, &lChecksum, sizeof(lChecksum), &dwBytesWritten, NULL);

	//Write the Application Version Checksum.
	lChecksum = MakeChecksum(COMP_VERSION);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&lChecksum, sizeof(lChecksum));
	WriteFile(hfTarget, &lChecksum, sizeof(lChecksum), &dwBytesWritten, NULL);

	//Write the Password Checksum.
	lChecksum = MakeChecksum(sUserKey);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&lChecksum, sizeof(lChecksum));
	DfltCode.CipherEx(&lChecksum, &lChecksum, sizeof(lChecksum));
	WriteFile(hfTarget, &lChecksum, sizeof(lChecksum), &dwBytesWritten, NULL);

	//------------------------------------------------------------------

	//Write the Package Attributes.
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&iCertAttribs, sizeof(iCertAttribs));
	DfltCode.CipherEx(&iCertAttribs, &iCryptTemp, sizeof(iCryptTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);

	//Write the Cryptography Algorithm
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&iCryptAlgo, sizeof(iCryptAlgo));
	DfltCode.CipherEx(&iCryptAlgo, &iCryptTemp, sizeof(iCryptAlgo));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);

	//Write the Julian Creation Date.
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&dwCreationDate, sizeof(dwCreationDate));
	DfltCode.CipherEx(&dwCreationDate, &dwCreationDate, sizeof(dwCreationDate));
	WriteFile(hfTarget, &dwCreationDate, sizeof(dwCreationDate), &dwBytesWritten, NULL);

	//Write the Julian Creation Time.
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&dwCreationTime, sizeof(dwCreationTime));
	DfltCode.CipherEx(&dwCreationTime, &dwCreationTime, sizeof(dwCreationTime));
	WriteFile(hfTarget, &dwCreationTime, sizeof(dwCreationTime), &dwBytesWritten, NULL);

	//Write the File thumb print or serial number.
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sThumbPrint, THUMB_PRINT_SZ);
	DfltCode.CipherEx(sThumbPrint, sThumbPrint, THUMB_PRINT_SZ);
	WriteFile(hfTarget, sThumbPrint, THUMB_PRINT_SZ, &dwBytesWritten, NULL);

	//Skip past the Reserved header space.
	SetFilePointer(hfTarget, RESERVE_HEADER_SZ, NULL, FILE_CURRENT);

	//------------------------------------------------------------------

	dwFinalCRC = EncryptCertKey(dwFinalCRC, hfTarget, &DfltCode, iCertKeySz);

	//------------------------------------------------------------------

	//Encrypt and write the sFirstName.
	iTemp = strlen(sFirstName);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sFirstName, iTemp);
	DfltCode.CipherEx(sFirstName, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sMiddleName.
	iTemp = strlen(sMiddleName);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sMiddleName, iTemp);
	DfltCode.CipherEx(sMiddleName, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sLastName.
	iTemp = strlen(sLastName);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sLastName, iTemp);
	DfltCode.CipherEx(sLastName, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sAddress1.
	iTemp = strlen(sAddress1);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sAddress1, iTemp);
	DfltCode.CipherEx(sAddress1, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sAddress2.
	iTemp = strlen(sAddress2);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sAddress2, iTemp);
	DfltCode.CipherEx(sAddress2, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sCity.
	iTemp = strlen(sCity);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sCity, iTemp);
	DfltCode.CipherEx(sCity, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sState.
	iTemp = strlen(sState);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sState, iTemp);
	DfltCode.CipherEx(sState, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sZipCode.
	iTemp = strlen(sZipCode);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sZipCode, iTemp);
	DfltCode.CipherEx(sZipCode, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sCountry.
	iTemp = strlen(sCountry);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sCountry, iTemp);
	DfltCode.CipherEx(sCountry, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sHomePhone.
	iTemp = strlen(sHomePhone);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sHomePhone, iTemp);
	DfltCode.CipherEx(sHomePhone, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sWorkPhone.
	iTemp = strlen(sWorkPhone);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sWorkPhone, iTemp);
	DfltCode.CipherEx(sWorkPhone, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sCellPhone.
	iTemp = strlen(sCellPhone);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sCellPhone, iTemp);
	DfltCode.CipherEx(sCellPhone, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sFaxNumber.
	iTemp = strlen(sFaxNumber);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sFaxNumber, iTemp);
	DfltCode.CipherEx(sFaxNumber, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sEmailAddress.
	iTemp = strlen(sEmailAddress);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sEmailAddress, iTemp);
	DfltCode.CipherEx(sEmailAddress, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	//Encrypt and write the sComments.
	iTemp = strlen(sComments);
	DfltCode.CipherEx(&iTemp, &iCryptTemp, sizeof(iTemp));
	WriteFile(hfTarget, &iCryptTemp, sizeof(iCryptTemp), &dwBytesWritten, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sComments, iTemp);
	DfltCode.CipherEx(sComments, sCryptTemp, iTemp);
	WriteFile(hfTarget, sCryptTemp, iTemp, &dwBytesWritten, NULL);

	dwFinalCRC = (dwFinalCRC ^ 0xffffffff);

	DfltCode.CipherEx(&dwFinalCRC, &dwFinalCRC, sizeof(dwFinalCRC));
	WriteFile(hfTarget, &dwFinalCRC, sizeof(dwFinalCRC), &dwBytesWritten, NULL);

	//------------------------------------------------------------------

	CloseHandle(hfTarget);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void Truncate(char *sPrt, const int iPos)
{
	sPrt[iPos] = '\0';
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD DecryptCertKey(DWORD dwINFinalCRC, HANDLE hfSource, NASCCL *lpCode, int iCertKeySz)
{
	//Generate the certificate key.
	char sCertKey[CERTKEYBUFSZ + 1];

	DWORD dwKeySzLeft = (DWORD)iCertKeySz;
	DWORD dwBytesRead = 0;
	DWORD dwBufSz = CERTKEYBUFSZ;
	DWORD dwFinalCRC = dwINFinalCRC;
		
	while(dwKeySzLeft > 0)
	{
		if(dwKeySzLeft <= dwBufSz)
		{
			dwBufSz = dwKeySzLeft;
		}

		ReadFile(hfSource, sCertKey, dwBufSz, &dwBytesRead, NULL);

		dwKeySzLeft -= dwBytesRead;

		lpCode->CipherEx(sCertKey, sCertKey, dwBytesRead);		

		dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sCertKey, dwBytesRead);
	}

	return dwFinalCRC;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ReadCert(HWND hWnd, const char *sTarName, const char *sUserKey,
			   int *iOut_CertAttribs, int *iOut_CryptAlgo, int *iOut_CertKeySz,
			   char *sFirstName, char *sMiddleName, char *sLastName,
			   char *sAddress1, char *sAddress2,
			   char *sCity, char *sState, char *sZipCode, char *sCountry,
			   char *sHomePhone, char *sWorkPhone, char *sCellPhone, char *sFaxNumber,
			   char *sEmailAddress, char *sComments)
{
	DWORD dwCreationDate = 0;
	DWORD dwCreationTime = 0;
	DWORD dwBytesRead = 0;
	DWORD dwFinalCRC = 0xffffffff;
	DWORD dwCertCertificate = 0;

	long lChecksum = 0;

	int iTemp = 0;

	char sStatusText[1024];
	char sThumbPrint[THUMB_PRINT_SZ + 1];

	NASCCL DfltCode;

	memset(&DfltCode, 0, sizeof(DfltCode));

	HANDLE hfSource = CreateFile(
		sTarName,
		GENERIC_READ,
		0,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL
	);

	if(hfSource == INVALID_HANDLE_VALUE)
    {
        sprintf_s(sStatusText, sizeof(sStatusText),
			"Failed to open read file:\r\n'%s'",
			sTarName);

		MessageBox(hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
        
		CloseHandle(hfSource);

		return false;
    }

	//------------------------------------------------------------------

	DfltCode.InitializeCryptography(sUserKey);

	//GetSystemTime(&stCreation);

	//dwCreationDate = DateToLong(stCreation.wYear, stCreation.wMonth, stCreation.wDay);
	//dwCreationTime = TimeToLong(stCreation.wHour, stCreation.wMinute, stCreation.wSecond);

	//GenerateKey(THUMB_PRINT_SZ, dwCreationDate + dwCreationTime, GKNONREPETITION | GKBROADKEY, sThumbPrint);

	//------------------------------------------------------------------

	//Write the Application Checksum.
	ReadFile(hfSource, &lChecksum, sizeof(lChecksum), &dwBytesRead, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&lChecksum, sizeof(lChecksum));
	if(lChecksum != MakeChecksum(APP_IDENTITY))
	{
		MessageBox(hWnd,
			"The file you are trying to decrypt was not encrypted with this application.",
			gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);

		CloseHandle(hfSource);

		return false;
	}

	//Write the Application Version Checksum.
	ReadFile(hfSource, &lChecksum, sizeof(lChecksum), &dwBytesRead, NULL);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&lChecksum, sizeof(lChecksum));
	if(lChecksum != MakeChecksum(COMP_VERSION))
	{
		MessageBox(hWnd,
			"The file you are trying to decrypt was encrypted using an incompatible version.",
			gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);

		CloseHandle(hfSource);

		return false;
	}

	//Write the Password Checksum.
	ReadFile(hfSource, &lChecksum, sizeof(lChecksum), &dwBytesRead, NULL);
	DfltCode.CipherEx(&lChecksum, &lChecksum, sizeof(lChecksum));
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&lChecksum, sizeof(lChecksum));
	if(lChecksum != MakeChecksum(sUserKey))
	{
		MessageBox(hWnd,
			"Wrong key, Can't decrypt.",
			gsTitleCaption, MB_ICONINFORMATION|MB_OK|MB_APPLMODAL);

		CloseHandle(hfSource);

		return false;
	}

	//------------------------------------------------------------------

	int iCertAttribs = 0;

	//Write the Package Attributes.
	ReadFile(hfSource, &iCertAttribs, sizeof(iCertAttribs), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iCertAttribs, &iCertAttribs, sizeof(iCertAttribs));
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&iCertAttribs, sizeof(iCertAttribs));

	int iCryptAlgo = 0;

	//Write the Cryptography Algorithm
	ReadFile(hfSource, &iCryptAlgo, sizeof(iCryptAlgo), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iCryptAlgo, &iCryptAlgo, sizeof(iCryptAlgo));
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&iCryptAlgo, sizeof(iCryptAlgo));

	//Write the Julian Creation Date.
	ReadFile(hfSource, &dwCreationDate, sizeof(dwCreationDate), &dwBytesRead, NULL);
	DfltCode.CipherEx(&dwCreationDate, &dwCreationDate, sizeof(dwCreationDate));
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&dwCreationDate, sizeof(dwCreationDate));

	//Write the Julian Creation Time.
	ReadFile(hfSource, &dwCreationTime, sizeof(dwCreationTime), &dwBytesRead, NULL);
	DfltCode.CipherEx(&dwCreationTime, &dwCreationTime, sizeof(dwCreationTime));
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)&dwCreationTime, sizeof(dwCreationTime));

	//Write the File thumb print or serial number.
	ReadFile(hfSource, sThumbPrint, THUMB_PRINT_SZ, &dwBytesRead, NULL);
	DfltCode.CipherEx(sThumbPrint, sThumbPrint, THUMB_PRINT_SZ);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sThumbPrint, THUMB_PRINT_SZ);
	Truncate(sThumbPrint, THUMB_PRINT_SZ);

	//Skip past the Reserved header space.
	SetFilePointer(hfSource, RESERVE_HEADER_SZ, NULL, FILE_CURRENT);

	int iCertKeySz = 0;

	//------------------------------------------------------------------
	//Encrypt and write the certificate key size.
	ReadFile(hfSource, &iCertKeySz, sizeof(iCertKeySz), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iCertKeySz, &iCertKeySz, sizeof(iCertKeySz));

	dwFinalCRC = DecryptCertKey(dwFinalCRC, hfSource, &DfltCode, iCertKeySz);

	//------------------------------------------------------------------

	//Encrypt and write the sFirstName.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sFirstName, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sFirstName, sFirstName, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sFirstName, iTemp);
	Truncate(sFirstName, iTemp);

	//Encrypt and write the sMiddleName.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sMiddleName, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sMiddleName, sMiddleName, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sMiddleName, iTemp);
	Truncate(sMiddleName, iTemp);

	//Encrypt and write the sLastName.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sLastName, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sLastName, sLastName, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sLastName, iTemp);
	Truncate(sLastName, iTemp);

	//Encrypt and write the sAddress1.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sAddress1, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sAddress1, sAddress1, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sAddress1, iTemp);
	Truncate(sAddress1, iTemp);

	//Encrypt and write the sAddress2.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sAddress2, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sAddress2, sAddress2, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sAddress2, iTemp);
	Truncate(sAddress2, iTemp);

	//Encrypt and write the sCity.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sCity, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sCity, sCity, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sCity, iTemp);
	Truncate(sCity, iTemp);

	//Encrypt and write the sState.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sState, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sState, sState, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sState, iTemp);
	Truncate(sState, iTemp);

	//Encrypt and write the sZipCode.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sZipCode, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sZipCode, sZipCode, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sZipCode, iTemp);
	Truncate(sZipCode, iTemp);

	//Encrypt and write the sCountry.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sCountry, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sCountry, sCountry, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sCountry, iTemp);
	Truncate(sCountry, iTemp);

	//Encrypt and write the sHomePhone.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sHomePhone, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sHomePhone, sHomePhone, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sHomePhone, iTemp);
	Truncate(sHomePhone, iTemp);

	//Encrypt and write the sWorkPhone.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sWorkPhone, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sWorkPhone, sWorkPhone, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sWorkPhone, iTemp);
	Truncate(sWorkPhone, iTemp);

	//Encrypt and write the sCellPhone.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sCellPhone, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sCellPhone, sCellPhone, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sCellPhone, iTemp);
	Truncate(sCellPhone, iTemp);

	//Encrypt and write the sFaxNumber.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sFaxNumber, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sFaxNumber, sFaxNumber, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sFaxNumber, iTemp);
	Truncate(sFaxNumber, iTemp);

	//Encrypt and write the sEmailAddress.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sEmailAddress, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sEmailAddress, sEmailAddress, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sEmailAddress, iTemp);
	Truncate(sEmailAddress, iTemp);

	//Encrypt and write the sComments.
	ReadFile(hfSource, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
	DfltCode.CipherEx(&iTemp, &iTemp, sizeof(iTemp));
	ReadFile(hfSource, sComments, iTemp, &dwBytesRead, NULL);
	DfltCode.CipherEx(sComments, sComments, iTemp);
	dwFinalCRC = PartialCRC(dwFinalCRC, (unsigned char *)sComments, iTemp);
	Truncate(sComments, iTemp);

	//------------------------------------------------------------------

	//Read the certificate checksum.
	ReadFile(hfSource, &dwCertCertificate, sizeof(dwCertCertificate), &dwBytesRead, NULL);
	DfltCode.CipherEx(&dwCertCertificate, &dwCertCertificate, sizeof(dwCertCertificate));

	CloseHandle(hfSource);

	if(dwCertCertificate != (dwFinalCRC ^ 0xffffffff))
	{
		MessageBox(hWnd,
			"This certificate has been modified or has become corrupt.",
			gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);

		return false;
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
