#ifndef _IndexDir_Cpp
#define _IndexDir_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <ShlOBJ.H>
#include <Stdlib.H>

#include "NSWFL.H"
#include "Entry.H"
#include "IndexDir.H"
#include "Common.H"

#include "../Dialogs/PackingDlg.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FilePack::CalcFileSizes(LPFILEPACKAGE lpFilePackage)
{
    DWORD dwLoop = 0;
	DWORD dwFileSz = 0;

	bool bReturnVal = true;

	lpFilePackage->dwTotalSize = 0;

	while(dwLoop < lpFilePackage->dwFileCount)
    {
		if((dwFileSz = Get_FileSize(lpFilePackage->FileNames[dwLoop])) == 0xFFFFFFFF)
		{
			bReturnVal = false;
		}

		lpFilePackage->dwTotalSize += dwFileSz;

		dwLoop++;
	}

	return bReturnVal;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FilePack::FreeMemory(LPFILEPACKAGE lpFilePackage)
{
    DWORD dwLoop = 0;

	while(dwLoop < lpFilePackage->dwFileCount)
    {
		free(lpFilePackage->FileNames[dwLoop]);
		lpFilePackage->FileNames[dwLoop] = NULL;
		dwLoop++;
	}

    lpFilePackage->FileNames = NULL;
	lpFilePackage->dwTotalSize = 0;

    Allocs = 0;
    lpFilePackage->dwFileCount = 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FilePack::Initialize(LPFILEPACKAGE lpFilePackage)
{
    memset(lpFilePackage, 0, sizeof(lpFilePackage));
	
	lpFilePackage->FileNames = NULL;
	lpFilePackage->dwTotalSize = 0;

    Allocs = 0;
    lpFilePackage->dwFileCount = 0;

	if((lpFilePackage->FileNames = (char **) calloc(sizeof(char *), AllocIncrement + 1)) == NULL)
    {
        return false;
    }
    
    Allocs = AllocIncrement;

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int MakeRelativeRoot(char *inPath, char *outPath, int inPathLen, int inRootLen)
{
    int iRPos = inRootLen;
    int iWPos = 0;
    
    while(iRPos < inPathLen)
    {
        outPath[iWPos++] = inPath[iRPos++];
    }

    outPath[iWPos] = '\0';

    return iWPos;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FilePack::AddFile(char *inFolder, int RootLen, char *DirTemp, int DirLen, char *sFile, LPFILEPACKAGE lpFilePackage)
{
	if(lpFilePackage->dwFileCount == Allocs)
    {
        lpFilePackage->FileNames = (char **) realloc(lpFilePackage->FileNames, ((Allocs * sizeof(char *)) + (AllocIncrement * sizeof(char *))) + 1);
        Allocs = (Allocs + AllocIncrement);
    }

	int iNeededMemory = strlen(sFile) + (DirLen - RootLen) + 10;
	lpFilePackage->FileNames[lpFilePackage->dwFileCount] = (char *) calloc(iNeededMemory, 1);

    MakeRelativeRoot(inFolder, DirTemp, DirLen, RootLen);

	sprintf_s(lpFilePackage->FileNames[lpFilePackage->dwFileCount],
		iNeededMemory, "%s\\%s", DirTemp, sFile);

	lpFilePackage->dwFileCount++;
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FilePack::IndexDirectoryInternal(char *inFolder, int RootLen, bool bRecursive, HWND hStatus, LPFILEPACKAGE lpFilePackage)
{
	WIN32_FIND_DATA FindData;
	HANDLE FileHandle;

	int Found = 0;

	char DirTemp[MAX_PATH];

	DWORD LocalLoop = 0;

	sprintf_s(DirTemp, sizeof(DirTemp), "%s\\%s", inFolder, "*.*");
	FileHandle = FindFirstFile(DirTemp,&FindData);
	Found = (int) FileHandle;

    int DirLen = strlen(inFolder);

	while(Found != 0)
	{
		if( strcmp(FindData.cFileName, ".") && strcmp(FindData.cFileName, "..") )
		{
			if(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				if(bRecursive)
				{
					sprintf_s(DirTemp, sizeof(DirTemp), "%s\\%s", inFolder, FindData.cFileName);

					SetStatusFileName(hStatus, FindData.cFileName);

					if(!IndexDirectoryInternal(DirTemp, RootLen, bRecursive, hStatus, lpFilePackage))
					{
						return false;
					}
				}
			}
			else {
				lpFilePackage->dwTotalSize += FindData.nFileSizeLow;

				if(!AddFile(inFolder, RootLen, DirTemp, DirLen, FindData.cFileName, lpFilePackage))
				{
					return false;
				}
			}
		}

		Found = FindNextFile(FileHandle, &FindData);
	}
	FindClose(FileHandle);

    if(gdwCurrentJobType == JOB_TYPE_CANCEL)
    {
        return false;
    }

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FilePack::IndexDirectory(char *inFolder, LPFILEPACKAGE lpFilePackage)
{
	lpFilePackage->dwTotalSize = 0;
	return IndexDirectoryInternal(inFolder, strlen(inFolder), true, NULL, lpFilePackage);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool FilePack::IndexDirectoryEx(char *inFolder, bool bRecursive, HWND hStatus, LPFILEPACKAGE lpFilePackage)
{
	lpFilePackage->dwTotalSize = 0;
	return IndexDirectoryInternal(inFolder, strlen(inFolder), bRecursive, hStatus, lpFilePackage);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
